#!/usr/bin/perl -w

package GOGraph::GeneProduct::SemanticSimilarity::GeneProductSimilarity;

=head1 NAME

  GOGraph::GeneProduct::SemanticSimilarity::GeneProductSimilarity

=head1 SYNOPSIS

  my $combined_similarity =
  GOGraph::GeneProduct::SemanticSimilarity::PairwiseAverageSimilarity
  ->new( GOGraph::Similarity::Factory::resnick_aspect_function_good() );

  my $gene_product_similarity =
  GOGraph::GeneProduct::SemanticSimilarity::GeneProductSimilarity
  ->new( $combined_similarity );

  my $similarity = $gene_product_similarity->semantic_similarity
  ( "opsr_human", "opsd_human" );

=head1 DESCRIPTION

  This class is a convienience class linking between the combined
  similarity classes and gene products. Given accession numbers or
  identifiers, depending on which is stored in the GO database, a
  similarity will be returned.

=head2 new

  Args - an instance of CombinedSimilarity
  Returns - an instance of this class

=head2 semantic_similarity

  Args - two strings representing accession number or identifiers
  Returns - a semantic similarity

=head1 AUTHOR

  Phillip Lord (p.lord@russet.org.uk)

=head1 COPYRIGHT

  This code is copyright by the University of Manchester (2002)

=head1 LICENSE

  This module is free software; you can redistribute it or
  modify it under the same terms as Perl itself.

  If you use this software for published research, please
  check my website (http://www.russet.org.uk) for recent
  citation.

=cut

use GOGraph::GOAppHandleFactory;

sub new{
  my $proto = shift;
  my $class = ref( $proto ) || $proto;
  my $combined_similarity = shift;

  my $self={};
  bless $self, $class;

  $self->{combined_similarity} = $combined_similarity;
  $self->{config} = Config::App::AppProperties::get_instance;
  $self->_ensure_handle;

  return $self;
}

sub _ensure_handle{
  my $self = shift;

  $self->{handle} = GOGraph::GOAppHandleFactory::go_apph();
}

sub semantic_similarity{
  my $self = shift;

  my $accession_one = lc(shift);
  my $accession_two = lc(shift);

  my $terms_one = $self->_fetch_terms( $accession_one );
  my $terms_two = $self->_fetch_terms( $accession_two );

  if( $terms_one == 0 ||
      $terms_two == 0 ){
    return -1;
  }

  return $self->{combined_similarity}->semantic_similarity
    ( $terms_one, $terms_two );
}


sub _fetch_terms{
  my $self = shift;
  my $accession = shift;

  my $apph = $self->{handle};

  $terms = $apph->get_terms({'product'=>$accession});

  unless( defined $terms ){
    $terms = 0;
  }

  return $terms;
}

1;
